"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("typedoc/dist/lib/models/index");
var plugin_options_1 = require("./plugin_options");
var typedoc_utils_1 = require("./typedoc_utils");
/**
 * Class that generates PlantUML code.
 */
var PlantUmlGenerator = /** @class */ (function () {
    /**
     * Creates a new PlantUmlGenerator object with the given options.
     * @param options The options for the PlantUML output.
     */
    function PlantUmlGenerator(options) {
        /**
         * Caches the PlantUML generated for the reflections (= class or interface).
         * KEY = ID of the reflection
         * VALUE = PlantUML lines
         */
        this.reflectionPlantUmlCache = new Map();
        this.options = options;
    }
    /**
     * Generates the Plant UML lines for the class diagram of the given reflection.
     * @param reflection The reflection for which to generate a class diagram.
     * @returns The Plant UML lines for the class diagram of the given reflection.
     *          If the given reflection is not part of an inheritance or implementation, the result is an empty array.
     */
    PlantUmlGenerator.prototype.createClassDiagramPlantUmlForReflection = function (reflection) {
        var includeMembers = this.options.umlClassDiagramType === plugin_options_1.ClassDiagramType.Detailed;
        var plantUmlLines = new Array();
        var siblingsAbove = 0;
        var siblingsBelow = 0;
        // add class/interface
        plantUmlLines = plantUmlLines.concat(this.getPlantUmlForReflection(reflection, includeMembers));
        // add classes/interfaces this type is extending
        var extendedTypes = typedoc_utils_1.TypeDocUtils.getExtendedTypesForReflection(reflection);
        for (var _i = 0, extendedTypes_1 = extendedTypes; _i < extendedTypes_1.length; _i++) {
            var type = extendedTypes_1[_i];
            plantUmlLines = plantUmlLines.concat(this.getPlantUmlForReflection(type, includeMembers));
            plantUmlLines.push(type.name + " <|-- " + reflection.name);
            ++siblingsAbove;
        }
        // add classes/interfaces this type is implementing
        var implementedTypes = typedoc_utils_1.TypeDocUtils.getImplementedTypesForReflection(reflection);
        for (var _a = 0, implementedTypes_1 = implementedTypes; _a < implementedTypes_1.length; _a++) {
            var type = implementedTypes_1[_a];
            plantUmlLines = plantUmlLines.concat(this.getPlantUmlForReflection(type, includeMembers));
            plantUmlLines.push(type.name + " <|.. " + reflection.name);
            ++siblingsAbove;
        }
        // add classes/interfaces that are extending this type
        var extendedBys = typedoc_utils_1.TypeDocUtils.getExtendedBysForReflection(reflection);
        for (var _b = 0, extendedBys_1 = extendedBys; _b < extendedBys_1.length; _b++) {
            var type = extendedBys_1[_b];
            plantUmlLines = plantUmlLines.concat(this.getPlantUmlForReflection(type, includeMembers));
            plantUmlLines.push(reflection.name + " <|-- " + type.name);
            ++siblingsBelow;
        }
        // add classes that are implementing this type
        var implementedBys = typedoc_utils_1.TypeDocUtils.getImplementedBysForReflection(reflection);
        for (var _c = 0, implementedBys_1 = implementedBys; _c < implementedBys_1.length; _c++) {
            var type = implementedBys_1[_c];
            plantUmlLines = plantUmlLines.concat(this.getPlantUmlForReflection(type, includeMembers));
            plantUmlLines.push(reflection.name + " <|.. " + type.name);
            ++siblingsBelow;
        }
        // Return no UML if there is no inheritance or implementation involved
        if (siblingsAbove + siblingsBelow === 0) {
            plantUmlLines = [];
        }
        else {
            if (this.options.umlClassDiagramHideEmptyMembers) {
                plantUmlLines.unshift("hide empty fields");
                plantUmlLines.unshift("hide empty methods");
            }
            if (this.options.umlClassDiagramHideCircledChar) {
                plantUmlLines.unshift("hide circle");
            }
            if (siblingsAbove > this.options.umlClassDiagramTopDownLayoutMaxSiblings ||
                siblingsBelow > this.options.umlClassDiagramTopDownLayoutMaxSiblings) {
                plantUmlLines.unshift("left to right direction");
            }
            if (this.options.umlClassDiagramMemberVisibilityStyle === plugin_options_1.ClassDiagramMemberVisibilityStyle.Text) {
                plantUmlLines.unshift("skinparam ClassAttributeIconSize 0");
            }
            if (this.options.umlClassDiagramHideShadow) {
                plantUmlLines.unshift("skinparam Shadowing false");
            }
            if (this.options.umlClassDiagramBoxBorderRadius) {
                plantUmlLines.unshift("skinparam RoundCorner " + this.options.umlClassDiagramBoxBorderRadius);
            }
            if (this.options.umlClassDiagramBoxBackgroundColor) {
                plantUmlLines.unshift("skinparam ClassBackgroundColor " + this.options.umlClassDiagramBoxBackgroundColor);
            }
            if (this.options.umlClassDiagramBoxBorderColor) {
                plantUmlLines.unshift("skinparam ClassBorderColor " + this.options.umlClassDiagramBoxBorderColor);
            }
            if (this.options.umlClassDiagramBoxBorderWidth >= 0) {
                plantUmlLines.unshift("skinparam ClassBorderThickness " + this.options.umlClassDiagramBoxBorderWidth);
            }
            if (this.options.umlClassDiagramArrowColor) {
                plantUmlLines.unshift("skinparam ClassArrowColor " + this.options.umlClassDiagramArrowColor);
            }
            if (this.options.umlClassDiagramClassFontName) {
                plantUmlLines.unshift("skinparam ClassFontName " + this.options.umlClassDiagramClassFontName);
            }
            if (this.options.umlClassDiagramClassFontSize) {
                plantUmlLines.unshift("skinparam ClassFontSize " + this.options.umlClassDiagramClassFontSize);
            }
            if (this.options.umlClassDiagramClassFontStyle !== plugin_options_1.FontStyle.Undefined) {
                plantUmlLines.unshift("skinparam ClassFontStyle " + this.options.umlClassDiagramClassFontStyle.toString());
            }
            if (this.options.umlClassDiagramClassFontColor) {
                plantUmlLines.unshift("skinparam ClassFontColor " + this.options.umlClassDiagramClassFontColor);
            }
            if (this.options.umlClassDiagramClassAttributeFontName) {
                plantUmlLines.unshift("skinparam ClassAttributeFontName " + this.options.umlClassDiagramClassAttributeFontName);
            }
            if (this.options.umlClassDiagramClassAttributeFontSize) {
                plantUmlLines.unshift("skinparam ClassAttributeFontSize " + this.options.umlClassDiagramClassAttributeFontSize);
            }
            if (this.options.umlClassDiagramClassAttributeFontStyle !== plugin_options_1.FontStyle.Undefined) {
                plantUmlLines.unshift("skinparam ClassAttributeFontStyle " +
                    this.options.umlClassDiagramClassAttributeFontStyle.toString());
            }
            if (this.options.umlClassDiagramClassAttributeFontColor) {
                plantUmlLines.unshift("skinparam ClassAttributeFontColor " + this.options.umlClassDiagramClassAttributeFontColor);
            }
        }
        return plantUmlLines;
    };
    /**
     * Returns an array of PlantUML lines for generating the box (including its properties and methods) of a given type.
     * Caches the result so that another call with the same reflection will be faster.
     * @param reflection The reflection for which the PlantUML should be generated.
     * @param includeMembers Specifies whether the resulting PlantUML should include the properties and methods of
     *                       the given reflection as well.
     * @returns The PlantUML lines for the given type.
     */
    PlantUmlGenerator.prototype.getPlantUmlForReflection = function (reflection, includeMembers) {
        if (!this.reflectionPlantUmlCache.has(reflection.id)) {
            this.reflectionPlantUmlCache.set(reflection.id, this.createPlantUmlForReflection(reflection, includeMembers));
        }
        return this.reflectionPlantUmlCache.get(reflection.id);
    };
    /**
     * Creates an array of PlantUML lines for generating the box (including its properties and methods) of a given type.
     * @param reflection The reflection for which the PlantUML should be generated.
     * @param includeMembers Specifies whether the resulting PlantUML should include the properties and methods of
     *                       the given reflection as well.
     * @returns The PlantUML lines for the given type.
     */
    PlantUmlGenerator.prototype.createPlantUmlForReflection = function (reflection, includeMembers) {
        var plantUmlLines = new Array();
        if (reflection.kind === index_1.ReflectionKind.Class || reflection.kind === index_1.ReflectionKind.Interface) {
            plantUmlLines.push(this.createPlantUmlForClassOrInterface(reflection) + " {");
            if (includeMembers && reflection.children) {
                for (var _i = 0, _a = reflection.children; _i < _a.length; _i++) {
                    var children = _a[_i];
                    if (children.kind === index_1.ReflectionKind.Property) {
                        plantUmlLines.push(this.createPlantUmlForProperty(children));
                    }
                    else if (children.kind === index_1.ReflectionKind.Method) {
                        plantUmlLines.push(this.createPlantUmlForMethod(children));
                    }
                }
            }
            plantUmlLines.push("}");
        }
        return plantUmlLines;
    };
    /**
     * Returns the PlantUML line for generating the output for a given property.
     * @param property The property for which the PlantUML should be generated.
     * @returns The PlantUML line for the given property.
     */
    PlantUmlGenerator.prototype.createPlantUmlForProperty = function (property) {
        var plantUml = "";
        if (property.flags.isStatic) {
            plantUml += "{static} ";
        }
        if (property.flags.isPrivate) {
            plantUml += "-";
        }
        else if (property.flags.isProtected) {
            plantUml += "#";
        }
        else {
            plantUml += "+"; // default is public for JS/TS
        }
        plantUml += property.name;
        if (property.type instanceof index_1.IntrinsicType ||
            property.type instanceof index_1.ReferenceType ||
            property.type instanceof index_1.TypeParameterType ||
            property.type instanceof index_1.UnknownType) {
            plantUml += " : " + property.type.name;
        }
        return plantUml;
    };
    /**
     * Returns the PlantUML line for generating the output for a given method.
     * @param methode The method for which the PlantUML should be generated.
     * @returns The PlantUML line for the given method.
     */
    PlantUmlGenerator.prototype.createPlantUmlForMethod = function (method) {
        var plantUml = "";
        if (method.flags.isStatic) {
            plantUml += "{static} ";
        }
        if (method.flags.isAbstract) {
            plantUml += "{abstract} ";
        }
        if (method.flags.isPrivate) {
            plantUml += "-";
        }
        else if (method.flags.isProtected) {
            plantUml += "#";
        }
        else {
            plantUml += "+"; // public is default for JS/TS
        }
        plantUml += method.name + "()";
        if (method.type) {
            if (method.type instanceof index_1.IntrinsicType ||
                method.type instanceof index_1.ReferenceType ||
                method.type instanceof index_1.TypeParameterType ||
                method.type instanceof index_1.UnknownType) {
                plantUml += " : " + method.type.name;
            }
        }
        else {
            plantUml += " : void";
        }
        return plantUml;
    };
    /**
     * Returns the PlantUML line for the introduction of a class or interface.
     * @param reflection The class or interface for which the PlantUML should be generated.
     * @returns The PlantUML line for the given class or interface.
     */
    PlantUmlGenerator.prototype.createPlantUmlForClassOrInterface = function (reflection) {
        var plantUml = "";
        if (reflection.flags.isStatic) {
            plantUml += "static ";
        }
        if (reflection.flags.isAbstract) {
            plantUml += "abstract ";
        }
        if (reflection.kind === index_1.ReflectionKind.Class) {
            plantUml += "class ";
        }
        else {
            plantUml += "interface ";
        }
        plantUml += reflection.name;
        return plantUml;
    };
    return PlantUmlGenerator;
}());
exports.PlantUmlGenerator = PlantUmlGenerator;
